<?php
/**
 * This is the entrypoint for the locrMAPS WordPress plugin.
 *
 * @package locrMAPS
 */

declare(strict_types=1);

namespace Locr\WordPress\Plugin\LocrMaps\DDD\Entities\OnlineMap;

use Locr\WordPress\Plugin\LocrMaps\DDD\ValueObjects\Icon\Id as IconId;
use Locr\WordPress\Plugin\LocrMaps\DDD\ValueObjects\OnlineMap\Location\{Address, Id, Position};

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * This class is responsible for an icon.
 *
 * @property-read Address $address The address.
 * @property-read Id $id The id.
 * @property-read IconId $icon_id The icon id.
 * @property-read Position $position The position.
 */
final class Location {
	/**
	 * The constructor.
	 *
	 * @param Id       $id The id.
	 * @param IconId   $icon_id The icon id.
	 * @param Position $position The position.
	 * @param Address  $address The address.
	 */
	public function __construct( private Id $id, private IconId $icon_id, private Position $position, private Address $address ) {
	}

	/**
	 * The magick getter.
	 *
	 * @param string $name The name of the property.
	 * @return mixed
	 * @throws \InvalidArgumentException If the property does not exist.
	 */
	public function __get( string $name ): mixed {
		return match ( $name ) {
			'address' => $this->address,
			'icon_id' => $this->icon_id,
			'id' => $this->id,
			'position' => $this->position,
			default => throw new \InvalidArgumentException( 'The property "' . esc_html( $name ) . '" does not exist.' ),
		};
	}

	/**
	 * Builds an icon instance from a database row.
	 *
	 * @param mixed $row The database row.
	 * @throws \InvalidArgumentException If the "id"-field is missing.
	 */
	public static function from_db_row( mixed $row ): self {
		if ( ! is_array( $row ) ) {
			throw new \InvalidArgumentException( 'The row must be an array.' );
		}
		if ( ! isset( $row['id'] ) ) {
			throw new \InvalidArgumentException( 'The "id"-field is missing from the database.' );
		}
		if ( ! isset( $row['icon_id'] ) ) {
			throw new \InvalidArgumentException( 'The "icon_id"-field is missing from the database.' );
		}
		if ( ! isset( $row['latitude'] ) ) {
			throw new \InvalidArgumentException( 'The "latitude"-field is missing from the database.' );
		}
		if ( ! isset( $row['longitude'] ) ) {
			throw new \InvalidArgumentException( 'The "longitude"-field is missing from the database.' );
		}
		if ( ! isset( $row['address'] ) ) {
			throw new \InvalidArgumentException( 'The "address"-field is missing from the database.' );
		}

		return new self(
			id: new Id( $row['id'] ),
			icon_id: new IconId( $row['icon_id'] ),
			position: new Position( $row['latitude'], $row['longitude'] ),
			address: Address::from_json( $row['address'] ),
		);
	}

	/**
	 * Converts the location to an array.
	 *
	 * @return array<mixed>
	 */
	public function to_array(): array {
		return array(
			'id'       => $this->id->value,
			'icon_id'  => $this->icon_id->value,
			'position' => array(
				'latitude'  => $this->position->latitude,
				'longitude' => $this->position->longitude,
			),
			'address'  => $this->address->to_array(),
		);
	}

	/**
	 * Converts the location to a JSON string.
	 *
	 * @throws \RuntimeException If the location could not be converted to JSON.
	 */
	public function to_json(): string {
		$json = wp_json_encode( $this->to_array() );
		if ( false === $json ) {
			throw new \RuntimeException( 'The location could not be converted to JSON.' );
		}

		return $json;
	}

	/**
	 * Updates the id.
	 *
	 * @param Id $id The id.
	 */
	public function update_id( Id $id ): self {
		$this->id = $id;
		return $this;
	}
}
